@@ -1,3 +1,5 @@
+"""Ascend resource applier classes - used for applying resource definitions."""
+
 import glog
 import networkx as nx
 from typing import List, Dict
@@ -11,22 +13,27 @@ from openapi_client.exceptions import ApiException
 
 
 class DataServiceApplier:
-  """DataServiceApplier is a utility class that accepts a DataService definition
-  and 'applies' it - ensuring that the DataService is created if it does not
-  already exist, and applying any configuration changes needed to match the supplied
-  definition.
+  """DataServiceApplier is a utility class that accepts a DataService definition and
+  'applies' it - ensuring that the DataService is created if it does not already exist,
+  deleting any existing Dataflows, components, and members of the DataService that
+  are not part of the supplied definition, and applying any configuration changes needed
+  to match the definition.
   """
   def __init__(self, client: Client):
+    """Creates a new DataServiceApplier."""
     self.client = client
 
   def apply(self, data_service: DataService, delete=True, dry_run=False):
     """
-    Create or update `data_service`.
-    :param data_service: DataService definition
-    :param delete: If set to True - delete any Dataflow that is not part of `data_service`.
-    At the Dataflow level, remove any components that are not part of the Dataflow definition.
-    :param dry_run: If set to True - skips any create or update operations
-    :return:
+    Create or update the specified DataService.
+
+    Parameters:
+    - data_service: DataService definition
+    - delete (optional): If set to `True` (which is the default) - delete any Dataflow
+    that is not part of `data_service`. At the Dataflow level, remove any components
+    that are not part of the Dataflow definition.
+    - dry_run (optional): If set to `True` (`False` by default) - skips any create or
+    update operations
     """
     glog.info(f"Apply DataService: {data_service.id}")
     exists = False
@@ -65,12 +72,14 @@ class DataServiceApplier:
 
 
 class DataflowApplier:
-  """DataflowApplier is a utility class that accepts a Dataflow definition
-  and 'applies' it - ensuring that a Dataflow is created if it does not already
-  exist, binding it to the DataService identified by `data_service_id`,
-  and applying any configuration changes needed to match the supplied definition.
+  """DataflowApplier is a utility class that accepts a Dataflow definition and 'applies'
+  it - ensuring that a Dataflow is created if it does not already exist and binding it to
+  the DataService identified by `data_service_id`, deleting any components and members of
+  the Dataflow that are not part of the supplied definition, and applying any configuration
+  changes needed to match the definition.
   """
   def __init__(self, client: Client):
+    """Creates a new DataflowApplier."""
     self.client = client
 
   def apply(self, data_service_id: str, dataflow: Dataflow, delete=True, dry_run=False):
@@ -79,12 +88,12 @@ class DataflowApplier:
     Dataflow definition - components (read connectors, transforms, write connectors), component
     groups, data feeds, and data feed connectors. The specified DataService must already exist.
 
-    :param data_service_id: DataService id
-    :param dataflow: Dataflow definition
-    :param delete: if set to True - delete any components, data feeds, data feed connectors,
-    or groups not defined in `dataflow`
-    :param dry_run: If set to True - skips any create or update operations
-    :return:
+    Parameters:
+    - data_service_id: DataService id
+    - dataflow: Dataflow definition
+    - delete: if set to `True` (default=`True`) - delete any components, data feeds,
+    data feed connectors, or groups not defined in `dataflow`
+    - dry_run: If set to `True` (default=`False`) - skips any create or update operations
     """
     self._apply_dataflow(data_service_id, dataflow, dry_run)
 
@@ -94,7 +103,7 @@ class DataflowApplier:
                                                              dry_run)
       id_map[dfc.id] = ComponentUuidType(type=dfc.legacy_type(), uuid=resource.uuid)
 
-    id_to_component = self._build_component_id_map(dataflow.components)
+    id_to_component = _component_id_map_from_list(dataflow.components)
     g = nx.DiGraph()
     for component in dataflow.components:
       g.add_node(component.id)
@@ -124,17 +133,6 @@ class DataflowApplier:
       if e.status != 404 or not dry_run:
         raise e
 
-  def _build_component_id_map(self, components: List[Component]) -> Dict[str, Component]:
-    id_to_component: Dict[str, Component] = {}
-    for component in components:
-      if not component.id:
-        raise ValueError(f"empty component id")
-      elif component.id in id_to_component:
-        raise ValueError(f"duplicate component id {component.id} in component list")
-      else:
-        id_to_component[component.id] = component
-    return id_to_component
-
   def _apply_dataflow(self, data_service_id: str, dataflow: Dataflow, dry_run=False):
     """ Create a dataflow if it does not already exist, otherwise update it.
     """
@@ -362,6 +360,10 @@ class ComponentApplier:
         self.client.get_write_connector(data_service_id, dataflow_id, component.id)
       elif isinstance(component, Transform):
         self.client.get_transform(data_service_id, dataflow_id, component.id)
+      else:
+        raise TypeError(
+            "Component has type {}, but expected one of: ReadConnector, WriteConnector, Transform".
+            format(type(component)))
       return True
     except ApiException as e:
       if e.status == 404:
@@ -433,6 +435,18 @@ class ComponentApplier:
     return ComponentApplier(client, id_map)
 
 
+def _component_id_map_from_list(components: List[Component]) -> Dict[str, Component]:
+  id_to_component: Dict[str, Component] = {}
+  for component in components:
+    if not component.id:
+      raise ValueError(f"empty component id")
+    elif component.id in id_to_component:
+      raise ValueError(f"duplicate component id {component.id} in component list")
+    else:
+      id_to_component[component.id] = component
+  return id_to_component
+
+
 def _component_id_to_uuid_map(client: Client, data_service_id: str,
                               dataflow_id: str) -> ComponentIdToUuidMap:
   id_map: ComponentIdToUuidMap = {}
