@@ -1,3 +1,6 @@
+"""Ascend resource definition classes."""
+
+import abc
 import google.protobuf.timestamp_pb2 as timestamp_pb2
 import google.protobuf.wrappers_pb2 as wrappers_pb2
 from typing import List, Dict
@@ -15,10 +18,15 @@ ComponentIdToUuidMap = Dict[str, ComponentUuidType]
 ComponentUuidToIdMap = Dict[str, str]
 
 
-class Component:
+class Definition(abc.ABC):
+  """Base class for all definitions."""
+  def __init__(self):
+    pass
+
+
+class Component(Definition):
   """Components are the functional units in a Dataflow.
-   There are three main types of Components: ReadConnectors, Transforms,
-   and WriteConnectors.
+   There are three main types of Components: ReadConnectors, Transforms, and WriteConnectors.
   """
   def __init__(self, id: str):
     self.id = id
@@ -48,19 +56,24 @@ class ReadConnector(Component):
                bytes: component_pb2.Source.FromBytes = None,
                records: component_pb2.Source.FromRecords = None):
     """
-    :param id: identifier that is unique within Dataflow
-    :param name: user-friendly name
-    :param description: brief description
-    :param container (proto): a typed representation of the source of data
-    :param pattern (proto): defines the set of objects in the source that will be ingested
-    :param update_periodical: specifies source update frequency
-    :param assigned_priority: scheduling priority for the read connector
-    :param last_manual_refresh_time:
-    :param aggregation_limit:
-    :param bytes: defines the operator for parsing bytes read from the source (example - json
+    Create a ReadConnector definition.
+
+    Parameters:
+    - id: identifier that is unique within Dataflow
+    - name: user-friendly name
+    - description: brief description
+    - container (proto): a typed representation of the source of data
+    - pattern (proto): defines the set of objects in the source that will be ingested
+    - update_periodical: specifies source update frequency
+    - assigned_priority: scheduling priority for the read connector
+    - last_manual_refresh_time:
+    - aggregation_limit:
+    - bytes: defines the operator for parsing bytes read from the source (example - json
       or xsv parsing)
-    :param records: defines the schema for records read from a database or warehouse
-    Note - a read connector can have either `bytes` or `records` defined, but not both
+    - records: defines the schema for records read from a database or warehouse
+    Returns: a new ReadConnector instance.
+
+    Note - a read connector can have either `bytes` or `records` defined, but not both.
     """
     super().__init__(id)
     self.name = name
@@ -136,12 +149,16 @@ class Transform(Component):
                description: str = None,
                assigned_priority: component_pb2.Priority = None):
     """
-    :param id: identifier that is unique within Dataflow
-    :param name: user-friendly name
-    :param description: brief description
-    :param input_ids: list of input component ids
-    :param operator: transform operator definition - example - SQL/PySpark/Scala transform
-    :param assigned_priority: scheduling priority for transform
+    Create a Transform definition.
+
+    Parameters:
+    - id: identifier that is unique within Dataflow
+    - name: user-friendly name
+    - description: brief description
+    - input_ids: list of input component ids
+    - operator: transform operator definition - example - SQL/PySpark/Scala transform
+    - assigned_priority: scheduling priority for transform
+    Returns: a new Transform instance.
     """
     super().__init__(id)
     self.name = name
@@ -202,16 +219,21 @@ class WriteConnector(Component):
                bytes: component_pb2.Sink.ToBytes = None,
                records: component_pb2.Sink.ToRecords = None):
     """
-    :param id: identifier that is unique within Dataflow
-    :param name: user-friendly name
-    :param description: brief description
-    :param input_id: input component id
-    :param container (proto): a typed representation of the destination of data
-    :param assigned_priority: scheduling priority of write connector
-    :param bytes: defines the operator for formatting bytes written to the object store destination
-    :param records: indicates that the destination is a structured record store like a database
+    Create a WriteConnector definition.
+
+    Parameters:
+    - id: identifier that is unique within Dataflow
+    - name: user-friendly name
+    - description: brief description
+    - input_id: input component id
+    - container (proto): a typed representation of the destination of data
+    - assigned_priority: scheduling priority of write connector
+    - bytes: defines the operator for formatting bytes written to the object store destination
+    - records: indicates that the destination is a structured record store like a database
     or a data warehouse
-    Note - a write connector can have either `bytes` or `records` defined, but not both
+    Returns: a new WriteConnector instance.
+
+    Note - a write connector can have either `bytes` or `records` defined, but not both.
     """
     super().__init__(id)
     self.name = name
@@ -268,7 +290,7 @@ class WriteConnector(Component):
       raise KeyError(f"unresolved id for component {e}")
 
 
-class DataFeed:
+class DataFeed(Definition):
   """A data feed"""
   def __init__(self,
                id: str,
@@ -279,16 +301,21 @@ class DataFeed:
                data_services_shared_with: List[str] = [],
                hidden_from_host_data_service: bool = False):
     """
-    :param id: identifier that is unique within Dataflow
-    :param name: user-friendly name
-    :param description: brief description
-    :param input_id: input component id
-    :param shared_with_all: if set to true, this data feed is shared with all data services,
+    Create a DataFeed definition.
+
+    Parameters:
+    - id: identifier that is unique within Dataflow
+    - name: user-friendly name
+    - description: brief description
+    - input_id: input component id
+    - shared_with_all: if set to true, this data feed is shared with all data services,
     including the host data service
-    :param data_services_shared_with: a list of ids of data services that we want to share
+    - data_services_shared_with: a list of ids of data services that we want to share
     data with (not including the host data service)
-    :param hidden_from_host_data_service: if set to true, the host data service cannot subscribe
+    - hidden_from_host_data_service: if set to true, the host data service cannot subscribe
     to the contents of this data feed
+    Returns: a new DataFeed instance.
+
     Note - if `shared_with_all` is set to true, the values of `data_services_shared_with` and
     `hidden_from_host_data_service` are ignored
     """
@@ -353,7 +380,7 @@ class DataFeed:
       raise KeyError(f"unresolved uuid for component {e}")
 
 
-class DataFeedConnector:
+class DataFeedConnector(Definition):
   """A data feed connector"""
   def __init__(self,
                id: str,
@@ -363,12 +390,16 @@ class DataFeedConnector:
                input_data_feed_id: str,
                description: str = None):
     """
-    :param id: identifier that is unique within Dataflow
-    :param name: user-friendly name
-    :param description: brief description
-    :param input_data_service_id: id of the DataService that the input DataFeed belongs to
-    :param input_dataflow_id: id of the Dataflow that the input DataFeed belongs to
-    :param input_data_feed_id: input DataFeed id
+    Create a DataFeedConnector definition.
+
+    Parameters:
+    - id: identifier that is unique within Dataflow
+    - name: user-friendly name
+    - description: brief description
+    - input_data_service_id: id of the DataService that the input DataFeed belongs to
+    - input_dataflow_id: id of the Dataflow that the input DataFeed belongs to
+    - input_data_feed_id: input DataFeed id
+    Returns: a new DataFeedConnector instance.
     """
     self.id = id
     self.name = name
@@ -387,14 +418,18 @@ class DataFeedConnector:
     return "sub"
 
 
-class ComponentGroup:
+class ComponentGroup(Definition):
   """A component group"""
   def __init__(self, id: str, name: str, component_ids: List[str], description: str = None):
     """
-    :param id: identifier that is unique within Dataflow
-    :param name: user-friendly name
-    :param description: brief description
-    :param component_ids: list of ids of components that make up the group
+    Create a ComponentGroup definition.
+
+    Parameters:
+    - id: identifier that is unique within Dataflow
+    - name: user-friendly name
+    - description: brief description
+    - component_ids: list of ids of components that make up the group
+    Returns: a new ComponentGroup instance.
     """
     self.id = id
     self.name = name
@@ -425,16 +460,13 @@ class ComponentGroup:
       raise KeyError(f"unresolved id for component {e}")
 
 
-class Dataflow:
-  """ A Dataflow is a representation of the operations being
-  performed on data, expressed as a dependency graph. Operations
-  are expressed declaratively via components, and are broadly
-  categorized as read connectors (data ingestion), transforms
-  (transformation), and write connectors. Data feeds expose the
-  results of transforms to other dataflows and data services.
-  Data feed connectors are subscriptions to data feeds.
-  Component groups offer users a convenient way to organize
-  components that have a similar purpose.
+class Dataflow(Definition):
+  """ A Dataflow is a representation of the operations being performed on data, expressed
+  as a dependency graph. Operations are expressed declaratively via components, and are
+  broadly categorized as read connectors (data ingestion), transforms (transformation),
+  and write connectors. Data feeds expose the results of transforms to other dataflows
+  and data services. Data feed connectors are subscriptions to data feeds. Component groups
+  offer users a convenient way to organize components that have a similar purpose.
   """
   def __init__(self,
                id: str,
@@ -444,6 +476,19 @@ class Dataflow:
                data_feed_connectors: List[DataFeedConnector] = [],
                data_feeds: List[DataFeed] = [],
                groups: List[ComponentGroup] = []):
+    """
+    Create a Dataflow definition.
+
+    Parameters
+    - id: identifier for the Dataflow that is unique within its DataService
+    - name: user-friendly name
+    - description: brief description
+    - components: list of components
+    - data_feed_connectors: list of data feeds
+    - data_feeds: list of data feed connectors
+    - groups: list of component groups
+    Returns: a new Dataflow instance.
+    """
     self.id = id
     self.name = name
     self.description = description
@@ -456,13 +501,22 @@ class Dataflow:
     return api_pb2.Dataflow(id=self.id, name=self.name, description=self.description)
 
 
-class DataService:
-  """A DataService is the highest-level organizational structure in
-  Ascend, and the primary means of controlling access.
-  It contains one or more Dataflows and can communicate with other
-  Data Services via Data Feeds.
+class DataService(Definition):
+  """A DataService is the highest-level organizational structure in Ascend, and the primary
+  means of controlling access. It contains one or more Dataflows and can communicate with
+  other Data Services via Data Feeds.
   """
   def __init__(self, id: str, name: str, description: str = None, dataflows: List[Dataflow] = []):
+    """
+    Create a DataService definition.
+
+    Parameters:
+    - id: DataService identifier
+    - name: name
+    - description: brief description
+    - dataflows: list of dataflows
+    Returns: a new DataService instance.
+    """
     self.id = id
     self.name = name
     self.description = description
